use strict;
use warnings;
use Collectd qw( :all );

# Initial Author: Adnane RABIH
# Refactored by: Hanine HAMZIOUI
# TODO no need for the mx4j
# TODO additional stats are eventually needed: gcstats / tpstats

# UTIL FUNCTION
no warnings 'all'; # ?

my $plugin_name = "cass"; # Plugin Name
my $nodeip = `ip route | grep -E '^10\.' | awk '{print \$1}' | sed -e 's/\/.*//g'`;
chomp($nodeip);

# First seed in cassandra configuration. We can use magicip local as a fallback but that is risky
my $casseed = `cat /etc/cassandra/conf/cassandra.yaml | grep seeds | cut -d':' -f2 | cut -d',' -f1 | sed -e "s/'//g" -e "s/ //g" 2>/dev/null`;
chomp($casseed);
if ($casseed eq '') {
    $casseed = $nodeip;
}
my $mxpurl =  "${casseed}:8081";

################################ SYSTEM VALUE FETCHERS ###################

# Memory
sub check_cass_mem {
    my $HeapMemoryUsage = `curl http://$mxpurl/mbean?objectname=java.lang%3Atype%3DMemory -s|grep HeapMemoryUsage | grep -v NonHeapMemoryUsage` ;
    $HeapMemoryUsage =~ s/.*HeapMemoryUsage.*max=(\d+).*used=(\d+).*/$1,$2/;
    my @array_1 = split(',', $HeapMemoryUsage);
    my $NonHeapMemoryUsage = `curl http://$mxpurl/mbean?objectname=java.lang%3Atype%3DMemory -s| grep  NonHeapMemoryUsage` ;
    $NonHeapMemoryUsage =~ s/.*HeapMemoryUsage.*max=(\d+).*used=(\d+).*/$1,$2/;
    my @array_2 = split(',', $NonHeapMemoryUsage);
    chomp($array_1[1], $array_1[0], $array_2[0], $array_2[1]);
    my $data = {};
    # $data->{ 'HeapUsed' } = $array_1[1];
    my $totalheap = `/usr/bin/nodetool info | grep -i heap | grep -i memory | grep -vi off | awk '{print \$7}' | sed -e 's/://g' 2>/dev/null`;
    my $usedheap = `/usr/bin/nodetool info | grep -i heap | grep -i memory | grep -vi off | awk '{print \$5}' | sed -e 's/://g' 2>/dev/null`;
    chomp($totalheap); chomp($usedheap);
    my $heaprate = $usedheap / $totalheap * 100;
    $data->{ 'HeapUsed' } = $heaprate;
    $data->{ 'HeapMax' } = $array_1[0];
    $data->{ 'NonHeapUsed' } = $array_2[1];
    $data->{ 'NonHeapMax' } = $array_2[0];
    return $data;
}

# LOAD in GigaBytes. Data Size alike
sub check_cass_load {
    my $load = `/usr/bin/nodetool info | grep -i load | awk '{print \$3}' 2>/dev/null`;
    chomp($load);
    return $load;
}

# Exceptions count # 0 = normal
sub check_cass_exceptions {
    my $exceptions = `/usr/bin/nodetool info | grep -i exception | awk '{print \$3}' 2>/dev/null`;
    chomp($exceptions);
    return $exceptions;
}

# uptime
sub check_cass_uptime {
    my $Uptime = `/usr/bin/nodetool info | grep -i uptime | awk '{print \$4}' 2>/dev/null`;
    chomp($Uptime);
    return $Uptime;
}

# Node Status # Normal value = 0 . No connection = 1
sub check_cass_status {
    my $status = `/usr/bin/nodetool status | grep ConnectException | wc -l 2>/dev/null`;
    chomp($status);
    return $status;
}


sub check_cass_garbage {
    my $GarbageCollection = `curl http://$mxpurl/mbean?objectname=java.lang%3Atype%3DGarbageCollector%2Cname%3DConcurrentMarkSweep -s| grep CollectionCount` ;
    $GarbageCollection =~ s/.*CollectionCount.*class=\"mbean_row\"\>(\d+).*/$1/;
    return $GarbageCollection ;
}

sub check_cass_loadeclass {
    my $LoadedClasses = `curl  http://$mxpurl/mbean?objectname=java.lang%3Atype%3DClassLoading -s| grep TotalLoadedClassCount`;
    $LoadedClasses =~  s/.*TotalLoadedClassCount.*class=\"mbean_row\"\>(\d+).*/$1/;
    return $LoadedClasses ;
}

#################################### DISPATCHERS #######################################
sub dispatch_cass_mem
{
    my ($values_mem, $keys_mem, $type_mem) = @_;
    my $time = time;
    my $v_mem = {
        plugin   => $plugin_name,
        time     => $time,
        type_instance   => $keys_mem,
        values => $values_mem,
        type   => $type_mem
    };
    plugin_dispatch_values($v_mem);
    return 1;
}

sub dispatch_cass_uptime
{
    my ($values, $keys, $type) = @_;
    my $time = time;
    my $v = {
        plugin   => $plugin_name,
        time     => $time,
        type_instance   => $keys,
        values => $values,
        type   => $type
    };
    plugin_dispatch_values($v);
    return 1;
}

sub dispatch_cass_load
{
    my ($values, $keys, $type) = @_;
    my $time = time;
    my $v = {
        plugin   => $plugin_name,
        time     => $time,
        type_instance   => $keys,
        values => $values,
        type   => $type
    };
    plugin_dispatch_values($v);
    return 1;
}


sub dispatch_cass_exceptions
{
    my ($values, $keys, $type) = @_;
    my $time = time;
    my $v = {
        plugin   => $plugin_name,
        time     => $time,
        type_instance   => $keys,
        values => $values,
        type   => $type
    };
    plugin_dispatch_values($v);
    return 1;
}


sub dispatch_cass_status
{
    my ($values, $keys, $type) = @_;
    my $time = time;
    my $v = {
        plugin   => $plugin_name,
        time     => $time,
        type_instance   => $keys,
        values => $values,
        type   => $type
    };
    plugin_dispatch_values($v);
    return 1;
}

sub dispatch_cass_garbage
{
    my ($values, $keys, $type) = @_;
    my $time = time;
    my $v = {
        plugin   => $plugin_name,
        time     => $time,
        type_instance   => $keys,
        values => $values,
        type   => $type
    };
    plugin_dispatch_values($v);
    return 1;
}

sub dispatch_cass_loadeclass
{
    my ($values, $keys, $type) = @_;
    my $time = time;
    my $v = {
        plugin   => $plugin_name,
        time     => $time,
        type_instance   => $keys,
        values => $values,
        type   => $type
    };
    plugin_dispatch_values($v);
    return 1;
}

#################################### VALUES READER #################################
sub read_cass {
    my $data = check_cass_mem();
    while ( my ($keys_mem, $values_mem) = each(%$data) ) {
        dispatch_cass_mem([$values_mem], "$keys_mem", 'cass_memory');
    }

    my $garbage = check_cass_garbage();
    dispatch_cass_garbage([$garbage], 'garbage_collector_count', 'cass_garbage');

    my $loadedclasses = check_cass_loadeclass();
    dispatch_cass_loadeclass([$loadedclasses], 'loaded_classes', 'cass_loaded_classes');

    my $load = check_cass_load();
    dispatch_cass_load([$load], 'load', 'cass_load');

    my $status = check_cass_status();
    dispatch_cass_status([$status], 'status', 'cass_status');

    my $exceptions = check_cass_exceptions();
    dispatch_cass_load([$exceptions], 'exceptions', 'cass_exceptions');

    my $uptime = check_cass_uptime();
    dispatch_cass_uptime([$uptime], 'uptime', 'cass_uptime');


    return 1;
}

plugin_register(TYPE_READ, $plugin_name, "read_cass");
1;
